import { AggregateCommit, EventStore } from '@ddes/core';
import { DynamoDB } from 'aws-sdk';
/**
 * Interface for EventStore powered by DynamoDB
 */
export declare class DynamodbEventStore extends EventStore {
    protected tableName: string;
    protected initialCapacity: StoreCapacityConfig;
    protected autoscaling?: AutoscalingConfig;
    protected client: DynamoDB;
    constructor(config: {
        tableName: string;
        client: DynamoDB;
        initialCapacity?: StoreCapacityConfig;
        autoscaling?: AutoscalingConfig;
    });
    /**
     * Create DynamoDB table and auto scaling configuration
     */
    setup(): Promise<void>;
    /** @inheritdoc */
    teardown(): Promise<void>;
    /**
     * @inheritdoc
     */
    commit<TAggregateCommit extends AggregateCommit>(commit: TAggregateCommit): Promise<TAggregateCommit>;
    /** @inheritdoc */
    getAggregateHeadCommit<TAggregateCommit extends AggregateCommit>(type: string, key: string): Promise<TAggregateCommit | undefined>;
    /**
     * Retrieve commits from the store chronologically
     */
    chronologicalQuery<TAggregateCommit extends AggregateCommit>(params: {
        group?: string;
        min: string | Date;
        max?: string | Date;
        exclusiveMin?: boolean;
        exclusiveMax?: boolean;
        descending?: boolean;
        limit?: number;
        timeDriftCompensation?: number;
        aggregateTypes?: string[];
    }): AsyncGenerator<TAggregateCommit[], void, unknown>;
    /**
     * Retrieve ordered commits for each aggregate instance of [[AggregateType]]
     */
    scanAggregateCommitsGroupedByKey<TAggregateCommit extends AggregateCommit>(type: string): AsyncGenerator<TAggregateCommit[], void, unknown>;
    /**
     * Query the commits of an [[Aggregate]] instance
     */
    queryAggregateCommits<TAggregateCommit extends AggregateCommit>(type: string, key: string, options?: {
        consistentRead?: boolean;
        minVersion?: number;
        maxVersion?: number;
        maxTime?: Date | number;
        descending?: boolean;
        limit?: number;
    }): AsyncGenerator<TAggregateCommit[], void, unknown>;
    /**
     * Scan store commits
     */
    scan<TAggregateCommit extends AggregateCommit>(params?: {
        totalSegments?: number;
        segment?: number;
        keyExpressions?: string[];
        filterExpressions?: string[];
        aggregateTypes?: string[];
        queryVariables?: object;
    }): AsyncGenerator<TAggregateCommit[], void, unknown>;
    chronologicalKey(data: {
        aggregateType: string;
        aggregateKey: string;
        aggregateVersion: number;
        timestamp: number;
    }): string;
    protected marshallCommit(commit: AggregateCommit): MarshalledCommit;
    protected unmarshallCommit<TAggregateCommit extends AggregateCommit>(marshalledCommit: Record<string, any>): TAggregateCommit;
    protected get tableSpecification(): {
        TableName: string;
        AttributeDefinitions: {
            AttributeName: string;
            AttributeType: string;
        }[];
        KeySchema: {
            AttributeName: string;
            KeyType: string;
        }[];
        ProvisionedThroughput: {
            ReadCapacityUnits: number;
            WriteCapacityUnits: number;
        };
        GlobalSecondaryIndexes: ({
            IndexName: string;
            KeySchema: {
                AttributeName: string;
                KeyType: string;
            }[];
            Projection: {
                ProjectionType: string;
                NonKeyAttributes: string[];
            };
            ProvisionedThroughput: {
                ReadCapacityUnits: number;
                WriteCapacityUnits: number;
            };
        } | {
            IndexName: string;
            KeySchema: {
                AttributeName: string;
                KeyType: string;
            }[];
            Projection: {
                ProjectionType: string;
                NonKeyAttributes?: undefined;
            };
            ProvisionedThroughput: {
                ReadCapacityUnits: number;
                WriteCapacityUnits: number;
            };
        })[];
    };
    protected request(type: 'scan', params: {
        filterExpressions?: string[];
        limit?: number;
    } & Partial<DynamoDB.ScanInput>): AsyncGenerator<DynamoDB.ScanOutput>;
    protected request(type: 'query', params: {
        keyExpressions?: string[];
        filterExpressions?: string[];
        queryVariables?: object;
        limit?: number;
    } & Partial<DynamoDB.QueryInput>): AsyncGenerator<DynamoDB.QueryOutput>;
}
export interface AutoscalingConfig {
    tableReadMin: number;
    tableReadMax: number;
    tableWriteMin: number;
    tableWriteMax: number;
    tableScaleInCooldown: number;
    tableScaleOutCooldown: number;
    chronologicalReadMin: number;
    chronologicalReadMax: number;
    chronologicalWriteMin: number;
    chronologicalWriteMax: number;
    chronologicalScaleInCooldown: number;
    chronologicalScaleOutCooldown: number;
    instancesReadMin: number;
    instancesReadMax: number;
    instancesWriteMin: number;
    instancesWriteMax: number;
    instancesScaleInCooldown: number;
    instancesScaleOutCooldown: number;
    utilizationTargetInPercent: number;
}
export interface StoreCapacityConfig {
    tableRead: number;
    tableWrite: number;
    chronologicalRead: number;
    chronologicalWrite: number;
    instancesRead: number;
    instancesWrite: number;
}
interface MarshalledCommit extends DynamoDB.AttributeMap {
    /**
     * Aggregate stream id type e.g. 'Order:123' (table partition key)
     */
    s: {
        S: string;
    };
    /**
     * Aggregate version (table sort key)
     */
    v: {
        N: string;
    };
    /**
     * Chronological sort key (chronological index sort key)
     */
    g: {
        S: string;
    };
    /**
     * Aggregate type
     */
    a: {
        S: string;
    };
    /**
     * Aggregate root commit key (only set for version = 1 commits)
     */
    r: {
        S: string;
    };
    /**
     * Commit timestamp
     */
    t: {
        N: string;
    };
    /**
     * Events in JSON form
     */
    e: {
        B: string;
    };
    /**
     * TTL timestamp (commit will be deleted at the set time)
     */
    x: {
        N: string;
    };
    /**
     * Chronological index partition key
     *
     */
    p: {
        S: string;
    };
}
export {};
